#!ruby
#**********************************************************************
# ruby p1_chknewplayer.rb YEAR CSVFILE
#----------------------------------------------------------------------
# 選手コードを付与した大会結果を作成する
# 選手が成績名簿マスターに登録されているか確認する。
# 同姓同名選手がいる場合は名前をコードに置換する。
# 入力
#    YEAR     集計対象の西暦年
#    CSVFILE  大会結果CSVファイル
# 出力
#   FILE_PLAYERS    大会出場した選手の成績名簿
#   FILE_RESULT     選手コードを付与した大会結果CSVファイル
#   FILE_RESULT1    FILE_RESULTのバックアップ
#   FILE_SAMENAME   同姓同名選手の抽出用
#   FILE_RESULT2    同姓同名選手を選手コード化対応した大会結果CSVファイル
#----------------------------------------------------------------------

require 'fileutils'
require './def_rating_files'
require './def_rating_meibo'


#######################################################################
# メインプログラム

#######################################################################
# 引数チェック
# p1: year     ex) 2020
# p2: csvfile  ex) 20200126-007-大阪.csv

errmes = "rating_sub.rb から実行してください。"

argc = ARGV.count
if argc != 2
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end

year = ARGV[0]
if year !~ /^[0-9]+$/
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end

csvfile = ARGV[1]
if csvfile !~ /^[0-9]+-[0-9]+-.+\.[cC][sS][vV]$/
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end


#######################################################################
# 準備

# 使用ディレクトリの準備を行う
wkdir   = RatingFiles.new(year)
tooldir = wkdir.rating_tooldir
Dir.chdir(tooldir)
date = wkdir.date_ymd_taikaicsv(csvfile)
csvdata = wkdir.get_csvdata(csvfile)

# 成績名簿マスターの使用準備を行う
wkmeibo = RatingMeibo.new
meibo = wkmeibo.get_meibo
wkmeibo.new_meibo_dousei_doumei
meibo_dd  = wkmeibo.get_meibo_dousei_doumei
meibo_ddk = wkmeibo.get_meibo_dousei_doumei_doukai

rank = RatingRanking.new


#######################################################################
# メインプログラム

#######################################################################
# 選手名・所属会により大会出場選手の成績名簿を作成する
# ・同姓同名選手を考慮していない。
# ・成績名簿マスターの選手コードの昇順でヒットした選手で作成する。

players_list   = []
result_list    = []
result2_list    = []
dd_result_list = []
unreg_player   = []
ddk_list       = []
result_list_cnt = 0

csvdata.each_line { |s|
	s.chomp!.sub!(/,+$/, "")
	p = s.split(/,/)
	p_name = p[0]
	p_kai = p[1]
	if p.length < 3
		STDERR.puts "対戦結果の記載がありません。"
		STDERR.puts s
		exit 1
	end
	
	findflag = 0
	meibo.each { |a|
		if p_kai == a[MB_KAI] && (p_name == a[MB_NAME] || p_name == a[MB_CODE])
			findflag = 1
			if rank.yuukou_kigen?(a[MB_DAY],date) == true
				# 持ち点有効期限内の選手
				players_list.push (a.join("\t") + "\t" + date)
			else
				# 持ち点有効期限切れの選手
				for i in 3..7 do a[i] = '0' end
				players_list.push (a.join("\t") + "\t" + date)
			end
			s1 = a[MB_CODE] + ',' + s
			result_list_cnt += 1
			result_list.push s1
			wkmeibo.dousei_doumei_name?(p_name) ? (dd_result_list.push s1) : nil
			break
		end
	}

	p_name_kai = "#{p_name},#{p_kai}"
	findflag == 0 ? (unreg_player.push p_name_kai) : nil
	wkmeibo.dousei_doumei_doukai_name?(p_name) ? (ddk_list.push p_name_kai) : nil
}

if unreg_player.length > 0
	File.open(FILE_ERROR, "w") { |f| 
		f.puts unreg_player
	}
	exit 1
end

if ddk_list.length > 0
	File.open(FILE_ERROR, "w") { |f|
		f.puts "同姓同名同会の選手が出場しています。"
		f.puts "該当の選手を選手名でなく選手コードで記述してください。"
		f.puts ddk_list
	}
	exit 1
end

players_list.length > 0 ? File.open(FILE_PLAYERS, "w") { |f| f.puts players_list } : nil
result_list.length > 0 ? File.open(FILE_RESULT, "w") { |f| f.puts result_list } : nil
dd_result_list.length > 0 ? File.open(FILE_SAMENAME, "w") { |f| f.puts dd_result_list } : nil
ddk_list.length > 0 ? File.open(FILE_SAMENAME2, "w") { |f| f.puts ddk_list } : nil
FileUtils.copy(FILE_RESULT, FILE_RESULT1)


#######################################################################
# 成績名簿マスター上の同姓同名選手の対応を行う
# ・所属会が異なることでシステムで判別できる場合は、対戦組合せの
#   クロスチェックにより選手コード化対応を行う。
# ・同姓同名同会のためにシステムで判別できない場合は、手作業による
#   選手コード化対応が必要になる。手作業を促すアラート表示を行う。

result_list = []
result2_list = []
f = File.open(FILE_RESULT, "r")
while s = f.gets
	s.chomp!
	a = s.split(/,/)
	result_list.push a
	result2_list.push s
end
f.close

dd_result_list.each { |s_dd|
	d = s_dd.split(/,/)

	# 対戦者のindex
	i = 3
	while i < d.length
		# 対戦相手欄の選手名をコード化する
		j = 0
		while j < result_list_cnt
			a = result_list[j]

			# 選手名をクロスチェックする(棄権マーク考慮あり)
			if i < a.length && \
					wkmeibo.eq_name?(a[RS_NAME],d[i]) == true && \
					wkmeibo.eq_name?(a[i],d[RS_NAME]) == true && \
					a[MB_CODE] != d[MB_CODE] then
				# 同姓同名の対戦者名をコード化した対戦結果
				a[i] = (a[i] =~ /\(GIVEN\)/) ? d[MB_CODE] + "(GIVEN)" : a[i]
				a[i] = (a[i] =~ /\(OFFER\)/) ? d[MB_CODE] + "(OFFER)" : d[MB_CODE]
				result_list[j] = a
				result2_list[j] = a.join(',')
				break
			end
			j += 1
		end
		i += 1
	end
}

dd_result_list.each { |s_dd|
	d = s_dd.split(/,/)
		# 選手名欄の選手名をコード化する
		j = 0
		while j < result_list_cnt
			a = result_list[j]
			if a[MB_CODE] == d[MB_CODE]
				a[MB_NAME] = d[MB_CODE]
				result_list[j] = a
				result2_list[j] = a.join(',')
				break
			end
			j += 1
		end
}

result_list.length > 0 ? File.open(FILE_RESULT2, "w") { |f| f.puts result2_list } : nil
FileUtils.copy(FILE_RESULT2, FILE_RESULT)

