#!ruby
#**********************************************************************
# ruby p3_syuukei.rb
#----------------------------------------------------------------------
# 対戦記録（FILE_GAME）からレーティング集計する
# 入力
#   FILE_PLAYERS    大会出場した選手の成績名簿
#   FILE_GAME       コード化された順序付き対戦記録
# 出力
#   FILE_SYUUKEI    レーティング集計結果
#   FILE_ERROR      エラー記録ファイル（エラー時）
#----------------------------------------------------------------------

require 'fileutils'
require './def_rating_files'
require './def_rating_meibo'


# レーティング曲線（ロジスティック曲線）のパラメータ
RC_ALPHA = 1.3
RC_BETA  = 0.4648
RC_GAMMA = 2689.0


###################################################################
# メソッド定義 kpt = init_kpoint(cnt_taikai, cnt_win)
# Kポイントを大会数、勝ち数から算出する
# 入力：
#  cnt_taikai : 大会数(Integer)
#  cnt_win    : 勝ち数(Integer)
# 戻り値：
#  kpt: Kポイント(Float)

	def init_kpoint(cnt_taikai, cnt_win)
		return 3.0 + cnt_win.to_f / (cnt_taikai.to_f + 1.0)
	end


###################################################################
# メソッド定義 kpt = kpoint(pt)
# Kポイントを持ち点から逆算する
# 入力：
#  pt : 持ち点(Integer)
# 戻り値：
#  kpt: Kポイント(Float)

	def kpoint(pt)
		return -Math.log((RC_GAMMA / pt.to_f - 1.0) / RC_ALPHA) / RC_BETA
	end


###################################################################
# メソッド定義 pt = mochiten(kpt)
# 持ち点をKポイントから算出する
# 入力：
#  kpt: Kポイント(Float)
# 戻り値：
#  pt : 持ち点(Integer)

	def mochiten(kpt = 0.0)
		return (RC_GAMMA / (1.0 + RC_ALPHA * Math.exp(-RC_BETA * kpt))).floor
	end


###################################################################
# メソッド定義 ptr = pnumber(pcode, p, pmax)
# 選手コードからリストポインタを求める。
# 入力：
#  pcode : 選手コード
#  p     : 選手成績集計リスト
#  pmax  : リストの最大データ個数
# 戻り値：
#  ptr   : 選手コードに対応するリストポインタ

	def pnumber(pcode = "", p = [], pmax)
		i = 1
		while i <= pmax
			ip = (i - 1) * SY_DNUM + MB_CODE
			if p[ip] == pcode
				return i
			end
			i += 1
		end
		return 0
	end


###################################################################
# メソッド定義 p1_mten, p2_mten = newdata(p1, p2, p)
# 新しい持ち点を算出する。
# 入力：
#  p1 : 勝者のリストポインタ
#  p2 : 敗者のリストポインタ
#  p  : 選手成績集計リスト
# 戻り値：
#  p1_mten : 勝者の持ち点
#  p2_mten : 敗者の持ち点

	def newdata(p1, p2, p = [])
		p[(p1 - 1) * SY_DNUM + SY_WIN] += 1			#勝者の勝数を加算
		p[(p2 - 1) * SY_DNUM + SY_LOS] += 1			#敗者の負数を加算

		p1T = p[(p1 - 1) * SY_DNUM + SY_TOUR]		#勝者の大会数
		p1W = p[(p1 - 1) * SY_DNUM + SY_WIN]		#勝者の勝ち数
		p2T = p[(p2 - 1) * SY_DNUM + SY_TOUR]		#敗者の大会数
		p2W = p[(p2 - 1) * SY_DNUM + SY_WIN]		#敗者の勝ち数

		ip1 = (p1 - 1) * SY_DNUM + SY_PT				#勝者の持ち点フィールド
		ip2 = (p2 - 1) * SY_DNUM + SY_PT				#敗者の持ち点フィールド

		p1P = p[ip1]														#勝者の持ち点
		p2P = p[ip2]														#敗者の持ち点

		#------------------------------------------------------------------
		# 対戦相手の両方が４大会を超えているとき、持ち点を計算する。
		#------------------------------------------------------------------
		if p1T > 4 && p2T > 4
			if p1P >= p2P
				d = (p1P - p2P) / 100.0
				t = (12.5 / (2 ** d)).floor / 100.0
			else
				d = (p2P - p1P) / 100.0
				t = (25 - 12.5 / (2 ** d)).floor / 100.0
			end

			k1 = kpoint(p1P) + t
			k2 = kpoint(p2P) - t
			p[ip1] = mochiten(k1)
			p[ip2] = mochiten(k2)
		end

		#------------------------------------------------------------------
		# 対戦相手のいずれかまたは両方が不定期間である場合は特別処理と
		# する。
		# 1998年3月現在の競技会方式における勝抜き度－ランクモデルからの
		# 推定値Ｋにより、持ち点の目安を計算する。
		# 持ち点初期値の上限値 2450 (2007/9/24導入)
		#------------------------------------------------------------------
		uplimit = 2450
		if p1T <= 4
			k1 = init_kpoint(p1T, p1W)
			p[ip1] = mochiten(k1)
			if p[ip1] > uplimit
				p[ip1] = uplimit
			end
		end
		if p2T <= 4
			k2 = init_kpoint(p2T, p2W)
			p[ip2] = mochiten(k2)
			if p[ip2] > uplimit
				p[ip2] = uplimit
			end
		end
		return p[ip1], p[ip2]
	end


#######################################################################
# メインプログラム

#------------------------------------------------------------------
# 対戦結果ファイルからリストへローディングする。
#------------------------------------------------------------------

err = 0
i = 0
p = []
pfile = []
date = ""

players = File.open(FILE_PLAYERS, "r")
players.each_line { |s|
	s.chomp!
	i += 1
	a = s.split(/\t/)

	p1 = a[MB_CODE]
	p2 = a[MB_NAME]
	p3 = a[MB_TOUR].to_i
	p4 = a[MB_WIN].to_i
	p5 = a[MB_LOS].to_i
	p6 = a[MB_NOP].to_i
	p7 = a[MB_PT].to_i

	p.push p1,p2,p3,p4,p5,p6,p7
	date = a[MB_TDAY]

	ip = (i - 1) * SY_DNUM
	p[ip + SY_TOUR] += 1
}
players.close
pmax = i


#------------------------------------------------------------------
# それぞれの対戦結果により、持ち点を集計する。
#------------------------------------------------------------------

game = File.open(FILE_GAME, "r")
while s = game.gets
	s.chomp!
	a = s.split(/\t/)
	if a.length != 3
		next
	end

	p1 = pnumber(a[1],p,pmax)
	if p1 == 0
		err = 1
		File.open(FILE_ERROR, "a") { |f|
			f.puts "#{s}"
			f.puts "  ERR> #{a[1]}"
		}
		exit err
	end

	p2 = pnumber(a[2],p,pmax)
	if a[2] == "-"
		ip = (p1 - 1) * SY_DNUM
		p[ip + SY_NOP] += 1
		next
	elsif p2 == 0
		err = 1
		File.open(FILE_ERROR, "a") { |f|
			f.puts "#{s}"
			f.puts "  ERR> #{a[2]}"
		}
		exit err
	else
		p1_mochiten = 0.0
		p2_mochiten = 0.0
		p1_mochiten, p2_mochiten = newdata(p1,p2,p)
		ip1 = (p1 - 1) * SY_DNUM
		ip2 = (p2 - 1) * SY_DNUM
		p[ip1 + SY_PT] = p1_mochiten
		p[ip2 + SY_PT] = p2_mochiten
	end

end
game.close


#------------------------------------------------------------------
# 集計結果をソートして、出力する。
#------------------------------------------------------------------

syuukei = []
i = 1
while i <= pmax
	ip = (i - 1) * SY_DNUM
	s = sprintf("%d\t%s\t%d\t%d\t%d\t%d\t%d\t%s",p[ip+0],p[ip+1],p[ip+2],p[ip+3],p[ip+4],p[ip+5],p[ip+6],date)
	syuukei.push s
	i += 1
end

# syuukeiを昇順に並べ替えて、FILE_SYUUKEIに出力する
syuukei.sort!
f = File.open(FILE_SYUUKEI, "w")
syuukei.each { |s|
	f.puts s
}
f.close

exit err

