#!ruby
#**********************************************************************
# ruby rating_sub.rb YEAR CSVFILE [-rank_display_on]
#----------------------------------------------------------------------
# 競技かるたレーティングシステム(Ruby版) サブプログラム
# 指定した大会結果CSVファイルのレーティング集計を行う。
# 入力
#   YEAR     集計対象の西暦年(yyyy)
#   CSVFILE  大会結果CSVファイル(yyyymmdd-nnn-name.csv)
#   -rank_display_on
#            集計時のランキング画面出力オンのオプション
#            内部用ランキングTXTファイル作成と連動する
#            (有:出力する、無:出力しない)
# 戻り値
#   0:正常終了、1:エラー
#----------------------------------------------------------------------
# 使用例：rating.rbからsystem()で呼ばれる。
#   ruby rating_sub.rb 2020 20200126-007-大阪.csv
#   ruby rating_sub.rb 2020 20200126-007-大阪.csv -rank_display_on
#----------------------------------------------------------------------

require 'nkf'
require 'fileutils'
require './def_rating_files'
require './def_rating_meibo'

#######################################################################
# メソッド定義 system_e(cmd, errcode)
# system(cmd)の実行結果が trueでない場合は、errcodeに対応するエラー
# 処理を行う。system(cmd)実行に伴うエラー処理出口を集約し、正常系処理の
# 流れの可読性向上をを目的としている。
# 入力
#   cmd      systemで実行するコマンド
#   errcode  エラーコード
# 戻り値
#   0:正常、1:エラー(通常は「exit 1」で中断終了する)
#----------------------------------------------------------------------

def system_e(cmd, errcode)
	if system(cmd) == true
		return 0
	end

	case errcode
		when "ERR_p1"
			if FileTest.exist?(FILE_ERROR) == true
				puts FILE_ERROR + " を確認します。"
				File.open(FILE_ERROR, "r") { |f| puts f.read }
			else
				if DEBUG_MODE == true
					puts "Command Error (ERR_p1) in " + __FILE__
					puts ""
				end
			end
			exit 1

		when "ERR_p2"
			if FileTest.exist?(FILE_ERROR) == true
				puts FILE_ERROR + " を確認します。"
				puts "大会結果CSVファイルのデータに誤りがあります。"
				File.open(FILE_ERROR, "r") { |f| puts f.read }
			else
				if DEBUG_MODE == true
					puts "Command Error (ERR_p2) in " + __FILE__
					puts ""
				end
			end
			exit 1

		when "ERR_p3"
			if FileTest.exist?(FILE_ERROR) == true
				puts FILE_ERROR + " を確認します。"
				puts "選手コードが " + FILE_PLAYERS + " に見つかりません。"
				File.open(FILE_ERROR, "r") { |f|
					puts f.read
				}
			else
				if DEBUG_MODE == true
					puts "Command Error (ERR_p3) in " + __FILE__
					puts ""
				end
			end
			exit 1

		when "ERR_p4"
			if DEBUG_MODE == true
				puts "Command Error (ERR_p4) in " + __FILE__
				puts ""
			end
			exit 1

		when "ERR_p5"
			if DEBUG_MODE == true
				puts "Command Error (ERR_p5) in " + __FILE__
				puts ""
			end
			exit 1

		when "ERR_p6"
			if DEBUG_MODE == true
				puts "Command Error (ERR_p6) in " + __FILE__
				puts ""
			end
			exit 1

		when "ERR_p7"
			if DEBUG_MODE == true
				puts "Command Error (ERR_p7) in " + __FILE__
				puts ""
			end
			exit 1
	end

	return 1
end


#######################################################################
# メソッド定義 dataform = csvfile_validation(csvdata)
# 大会結果CSVファイルの形式を確認する。
# 入力
#   csvdata   csvfileのファイル内容
# 戻り値
#   dataform  0:○×形式、1:一覧表形式
#   エラー時は「exit 1」で中断終了する。
#----------------------------------------------------------------------
# ○×形式と一覧表形式の判定方法
# ・'WIN'または'○'と'×'の有無を確認することで形式を判定する。
# ・一覧表形式：'○'を含まない。優勝者の行末データは'WIN'である。
# ・○×形式　：'○'と'×'を含む。'WIN'はない。
# ・一覧表形式、○×形式ともに次のものを使用しない。
#   半角空白、全角空白（分かち書きや文字列前後の空白を入力抑制する）
#   枚数差、'不戦勝'（'不戦勝'は'-'で表示する仕様）
#----------------------------------------------------------------------

def csvfile_validation(csvdata)
	# 半角空白を含む場合はエラー
	if csvdata.include?(" ") == true
		puts "大会結果CSVファイルのデータに誤りがあります。(半角空白の記載)"
		exit 1
	end
	
	# 全角空白を含む場合はエラー
	if csvdata.include?("　") == true
		puts "大会結果CSVファイルのデータに誤りがあります。(全角空白の記載)"
		exit 1
	end
	
	# 枚数差データがある場合はエラー
	if csvdata =~ /,[0-9]+/
		puts "大会結果CSVファイルのデータに誤りがあります。(枚差の記載)"
		exit 1
	end
	
	# '不戦勝'がある場合はエラー
	if csvdata.include?("不戦勝") == true
		puts "大会結果CSVファイルのデータに誤りがあります。(不戦勝の記載)"
		exit 1
	end
	
	# 'WIN''○''×'の組み合わせが不適切な場合はエラー
	flag_win = 0
	if csvdata =~ /WIN/
		flag_win = 1
	end
	
	flag_maru = 0
	if csvdata =~ /○/
		flag_maru = 1
	end
	
	flag_batsu = 0
	if csvdata =~ /×/
		flag_batsu = 1
	end
	
	if flag_win == 1 && flag_maru == 0 && flag_batsu == 0
		return 1	# 一覧表形式
	elsif flag_win == 0 && flag_maru == 1 && flag_batsu == 1
		return 0	# ○×形式
	elsif flag_win == 1 && (flag_maru == 1 || flag_batsu == 1)
		puts "大会結果CSVファイルのデータに誤りがあります。(WINと○×の両方の記載)"
		exit 1
	else
		puts "大会結果CSVファイルの'WIN'または'○×'に誤りがあります。"
		exit 1
	end

end


#######################################################################
# メインプログラム

#######################################################################
# 引数チェック
# ruby rating_sub.rb YEAR CSVFILE [-rank_display_on]
# p1: YEAR, p2: CSVFILE, p3: -rank_display_on

errmes = "rating.rb から実行してください。"

argc = ARGV.count
unless argc == 2 || argc == 3
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end

year = ARGV[0]
unless year =~ /^20[0-9][0-9]$/
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end

csvfile = ARGV[1]
unless csvfile =~ /^[0-9]+-[0-9]+-.+\.[cC][sS][vV]$/
	STDERR.puts errmes
	STDERR.puts ""
	exit 1
end

p3 = ARGV[2]
if p3 == "-rank_display_on"
	rank_display_on = 1		# 画面出力する
else
	rank_display_on = 0		# 画面出力しない
end


#######################################################################
# 集計準備後、スタートメッセージを表示する

# 使用ディレクトリの準備を行う
wkdir   = RatingFiles.new(year)
tooldir = wkdir.rating_tooldir
Dir.chdir(tooldir)
wkdir.save_meibo

# 集計前の一時作業ファイルを削除してデータクリアする
wkdir.delete_tempfile_before_syuukei

# 大会結果CSVファイルの内容取得及び形式確認する
csvdata = []
csvdata = wkdir.get_csvdata(csvfile)
dataform = csvfile_validation(csvdata)

# スタートメッセージ
puts "starting... #{csvfile}"


#######################################################################
# Phase-1. 選手コードを付与した大会結果を作成する
# 大会出場した選手がシステムに登録されているかを確認して
# 選手コードを大会結果に付与する。
# 入力
#   YEAR         集計対象の西暦年
#   CSVFILE      指定された大会結果CSVファイル
#   FILE_MEIBO   成績名簿マスター
# 出力
#   FILE_RESULT  選手コードが付与された大会結果

system_e("ruby p1_chknewplayer.rb #{year} #{csvfile}", "ERR_p1")

if File.size?(FILE_RESULT) > 0
	puts "phase 1. 選手の確認          -- OK"
else
	puts FILE_RESULT + " が作成されていません。"
	puts ""
	exit 1
end


#######################################################################
# Phase-2. 選手コードでコード化された順序付き対戦記録を作成する
# 入力
#   FILE_RESULT  選手コードが付与された大会結果
#   dataform     大会結果のデータ形式
# 出力
#   FILE_GAME    選手コードでコード化された順序付き対戦記録
#   この対戦記録をgameディレクトリにも作成し、持ち点初期値の補正に
#   用いる。

case dataform
	when 0 # ○×形式
		system_e("ruby p2_gamesort0.rb", "ERR_p2")
	when 1 # 一覧表形式
		system_e("ruby p2_gamesort1.rb", "ERR_p2")
end

if FileTest.exist?(FILE_GAME) == true && File.size?(FILE_GAME) > 0
	wkdir.save_game(csvfile)
	puts "phase 2. 対戦記録の確認      -- OK"
else
	puts FILE_GAME + " が作成されていません。"
	puts ""
	exit 1
end


#######################################################################
# Phase-3. 順序付き対戦記録からレーティング集計を行う

system_e("ruby p3_syuukei.rb", "ERR_p3")

if FileTest.exist?(FILE_SYUUKEI) == true && File.size?(FILE_SYUUKEI) > 0
	puts "phase 3. 大会成績の集計      -- OK"
else
	puts FILE_SYUUKEI + " が作成されていません。"
	puts ""
	exit 1
end


#######################################################################
# Phase-4. レーティング集計結果を成績名簿マスターへ反映する
# 入力
#   CSVFILE         指定された大会結果CSVファイル
#   FILE_SYUUKEI    レーティング集計結果
#   FILE_MEIBO      成績名簿マスター
# 出力
#   FILE_MERGE      集計結果がマージされた成績名簿マスター候補
#   成績名簿マスター候補をコピーして次のファイルを作成/更新する。
#     FILE_MEIBO      成績名簿マスター
#     FILE_MEIBO_BAK  成績名簿マスターの一時バックアップ
#     meibo/          成績名簿マスターの保存用

system_e("ruby p4_merge.rb #{csvfile}", "ERR_p4")

if FileTest.exist?(FILE_MERGE) == true && FileTest.size?(FILE_MERGE) > 0
	wkdir.baseline_meibo(csvfile)
	puts "phase 4. 大会成績の全体集計  -- OK"
else
	puts FILE_MERGE + " が作成されていません。"
	puts ""
	exit 1
end


#######################################################################
# レーティング集計の後処理. 大会結果の集計完了を記録する

# 集計済み大会結果を読み取り専用にする
if DEBUG_MODE == false
	puts csvfile + " を読み取り専用ファイルにします。"
	wkdir.readonly_taikaicsv(csvfile)
end

# 集計済み大会をYDBリストに追加する
wkdir.make_ydb
wkdir.add_taikai_to_ydb(csvfile)
wkdir.delete_file FILE_ERROR


#######################################################################
# Phase-5～Phase-7. ランキングを作成する

# ランキング対象の分類を行う
system_e("ruby p5_meibosorter.rb #{csvfile}", "ERR_p5")

if rank_display_on == 1
# 内部用ランキングTXTファイルを作成する(対象選手全員)
	puts "ランキング(" + FILE_RANKING + ")を作成します。"
	system_e("ruby p6_rankupdown.rb", "ERR_p6")
end

# 公開用ランキングHTMLファイルを作成する(対象選手Top200)
puts "ランキングTop200(" + FILE_RANKHTML + ")を作成します。"
system_e("ruby p7_rankhtml.rb", "ERR_p7")

htmlfile = csvfile.sub(/\.[cC][sS][vV]$/, ".htm")
File.rename(FILE_RANKHTML, htmlfile)

# ランキング作成後の一時作業ファイルを削除する
wkdir.delete_tempfile_after_ranking

puts "終了しました。"
exit 0

